@@ -1,12 +1,15 @@
-;;; quickrun.el --- Run commands quickly -*- lexical-binding: t; -*-
+;;; quickrun.el --- Run commands quickly  -*- lexical-binding: t; -*-
 
 ;; Copyright (C) 2017 by Syohei YOSHIDA
+;; Copyright (C) 2020-2026 by Jen-Chieh Shen
 
 ;; Author: Syohei YOSHIDA <syohex@gmail.com>
-;; URL: https://github.com/syohex/emacs-quickrun
-;; Package-Version: 2.3.1
-;; Package-Revision: 0133cac65191
-;; Package-Requires: ((emacs "24.3"))
+;; Maintainer: Jen-Chieh Shen <jcs090218@gmail.com>
+;; URL: https://github.com/emacsorphanage/quickrun
+;; Package-Version: 20260103.1800
+;; Package-Revision: 9199e222f951
+;; Package-Requires: ((emacs "26.1") (ht "2.0"))
+;; Keywords: tools
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -24,7 +27,7 @@
 ;;; Commentary:
 
 ;; quickrun.el executes editing buffer.  quickrun.el selects commands to execute
-;; buffer automatically.  Please see https://github.com/syohex/emacs-quickrun
+;; buffer automatically.  Please see https://github.com/emacsorphanage/quickrun
 ;; for more information.
 ;;
 ;; This package respects `quickrun.vim' developed by thinca
@@ -43,10 +46,14 @@
 (require 'em-banner)
 (require 'eshell)
 
+(require 'ht)
+
 ;; for warnings of byte-compile
 (declare-function anything "anything")
 (declare-function helm "helm")
 (declare-function tramp-dissect-file-name "tramp")
+(declare-function tramp-file-name-localname "tramp")
+(declare-function clear-image-cache "image")
 
 (defgroup quickrun nil
   "Execute buffer quickly."
@@ -63,6 +70,11 @@
   :type 'boolean
   :group 'quickrun)
 
+(defcustom quickrun-truncate-lines t
+  "The `truncate-lines' value for `*quickrun*` buffer."
+  :type 'boolean
+  :group 'quickrun)
+
 (defcustom quickrun-input-file-extension ".qrinput"
   "Extension of input file name."
   :type '(choice (string :tag "Extension of quickrun input file")
@@ -74,6 +86,11 @@
   :type 'boolean
   :group 'quickrun)
 
+(defcustom quickrun-output-only nil
+  "If non-nil, omit the header and footer from the output."
+  :type 'boolean
+  :group 'quickrun)
+
 (defconst quickrun--buffer-name "*quickrun*")
 (defvar quickrun--executed-file nil)
 (defvar quickrun--remove-files nil)
@@ -81,6 +98,10 @@
 (defvar quickrun--original-buffer nil)
 (defvar quickrun--original-outputter nil)
 
+(defun quickrun-2str (obj)
+  "Convert OBJ to string."
+  (format "%s" obj))
+
 (defmacro quickrun--awhen (test &rest body)
   "Anaphoric when.  If TEST is non-nil, do BODY (include `it')."
   (declare (indent 1))
@@ -115,24 +136,24 @@ FMT and ARGS passed `message'."
         `(put (quote ,name) 'safe-local-variable (quote ,safep)))))
 
 (quickrun--defvar quickrun-option-cmd-alist
-                  nil listp
-                  "Specify command alist directly as file local variable")
+  nil listp
+  "Specify command alist directly as file local variable")
 
 (quickrun--defvar quickrun-option-command
-                  nil stringp
-                  "Specify command directly as file local variable")
+  nil stringp
+  "Specify command directly as file local variable")
 
 (quickrun--defvar quickrun-option-cmdkey
-                  nil stringp
-                  "Specify language key directly as file local variable")
+  nil stringp
+  "Specify language key directly as file local variable")
 
 (quickrun--defvar quickrun-option-cmdopt
-                  nil stringp
-                  "Specify command option directly as file local variable")
+  nil stringp
+  "Specify command option directly as file local variable")
 
 (quickrun--defvar quickrun-option-args
-                  nil stringp
-                  "Specify command argument directly as file local variable")
+  nil stringp
+  "Specify command argument directly as file local variable")
 
 (defun quickrun--outputter-p (_x)
   "Not documented."
@@ -141,20 +162,20 @@ FMT and ARGS passed `message'."
         (quickrun--outputter-multi-p x))))
 
 (quickrun--defvar quickrun-option-outputter
-                  nil quickrun--outputter-p
-                  "Specify format function output buffer as file local variable")
+  nil quickrun--outputter-p
+  "Specify format function output buffer as file local variable")
 
 (quickrun--defvar quickrun-option-shebang
-                  t booleanp
-                  "Select using command from schebang as file local variable")
+  t booleanp
+  "Select using command from schebang as file local variable")
 
 (quickrun--defvar quickrun-option-timeout-seconds
-                  nil integerp
-                  "Timeout seconds as file local variable")
+  nil integerp
+  "Timeout seconds as file local variable")
 
 (quickrun--defvar quickrun-option-default-directory
-                  nil file-directory-p
-                  "Default directory where command is executed")
+  nil file-directory-p
+  "Default directory where command is executed")
 
 ;; hooks
 (defvar quickrun-after-run-hook nil
@@ -179,16 +200,35 @@ FMT and ARGS passed `message'."
   (image-mode))
 
 ;;
-;; language command parameters
-;;
+;;; language command parameters
 
 (defvar quickrun--language-alist
-  '(("c/gcc" . ((:command . "gcc")
+  '(("asm/masm" . ((:command . "ml")
+                   (:exec . ("%c /c /coff %s"
+                             "link /subsystem:windows %n.obj"
+                             "%n.exe"))
+                   (:compile-only . "%c /c /coff %s")
+                   (:remove  . ("%n.obj" "%n.exe"))
+                   (:description . "Compile Assembly file with masm and execute")))
+    ("asm/masm64" . ((:command . "ml64")
+                     (:exec . ("%c /c /coff %s"
+                               "link /subsystem:windows %n.obj"
+                               "%n.exe"))
+                     (:compile-only . "%c /c /coff %s")
+                     (:remove  . ("%n.obj" "%n.exe"))
+                     (:description . "Compile Assembly file with masm x64 and execute")))
+    ("asm/nasm" . ((:command . "nasm")
+                   (:exec . ("%c -f elf64 %o %s -o %e.o" "ld -o %e %e.o" "%e %a"))
+                   (:remove . ("%e" "%e.o"))
+                   (:description . "Compile Assembly file with nasm and execute")))
+    ("applescript" . ((:command . "osascript")
+                      (:description . "Run apple script")))
+
+    ("c/gcc" . ((:command . "gcc")
                 (:exec    . ("%c -x c %o -o %e %s" "%e %a"))
                 (:compile-only . "%c -Wall -Werror %o -o %e %s")
                 (:remove . ("%e"))
                 (:description . "Compile C file with gcc and execute")))
-
     ("c/clang" . ((:command . "clang")
                   (:exec    . ("%c -x c %o -o %e %s" "%e %a"))
                   (:compile-only . "%c -Wall -Werror %o -o %e %s")
@@ -273,9 +313,19 @@ FMT and ARGS passed `message'."
     ("php" . ((:command . "php") (:compile-only . "%c -l %s")
               (:description . "Run PHP script")))
 
-    ("emacs" . ((:command . "emacs")
-                (:exec    . "%c -q --no-site-file --batch -l %s")
-                (:description . "Run Elisp as script file")))
+    ("elisp/emacs" . ((:command . "emacs")
+                      (:exec    . "%c -q --no-site-file --batch -l %s")
+                      (:description . "Run Elisp as script file")))
+    ("elisp/eask" . ((:command . "eask load")
+                     (:exec    . "%c %s")
+                     (:description . "Run Elisp as script file through Eask (local scope)")))
+    ("elisp/eask-g" . ((:command . "eask load")
+                       (:exec    . "%c %s -g")
+                       (:description . "Run Elisp as script file through Eask (global scope)")))
+    ("elisp/eask-c" . ((:command . "eask load")
+                       (:exec    . "%c %s -c")
+                       (:description . "Run Elisp as script file through Eask (configuration scope)")))
+
     ("lisp/clisp" . ((:command . "clisp")
                      (:description . "Run Lisp file with clisp")))
     ("lisp/sbcl" . ((:command . "sbcl")
@@ -286,6 +336,9 @@ FMT and ARGS passed `message'."
                    (:description . "Run Lisp file with ccl")))
     ("scheme/gosh" . ((:command . "gosh")
                       (:description . "Run Scheme file with gosh(Gauche)")))
+    ("st/gst" . ((:command . "gst")
+                 (:exec . "%c -f %s %a")
+                 (:description . "Run Smalltalk file with GNU Smalltalk")))
     ("racket" . ((:command . "racket")
                  (:exec . "%c --require-script %s")
                  (:description . "Run racket script")))
@@ -296,19 +349,23 @@ FMT and ARGS passed `message'."
                               (:description . "Run Clojure file with clj-env-dir")))
 
     ("javascript/node" . ((:command . "node")
-                          (:description . "Run Javascript file with node.js")))
+                          (:description . "Run JavaScript file with Node.js")))
     ("javascript/v8" . ((:command . "v8")
-                        (:description . "Run Javascript file with v8")))
+                        (:description . "Run JavaScript file with v8")))
     ("javascript/js" . ((:command . "js")
-                        (:description . "Run Javascript file with js(Rhino)")))
+                        (:description . "Run JavaScript file with js(Rhino)")))
     ("javascript/jrunscript" . ((:command . "jrunscript")
-                                (:description . "Run Javascript file with jrunscript")))
+                                (:description . "Run JavaScript file with jrunscript")))
     ("javascript/phantomjs" . ((:command . "phantomjs")
-                               (:description . "Run Javascript file with phantomjs")))
+                               (:description . "Run JavaScript file with phantomjs")))
     ("javascript/cscript" . ((:command . "cscript")
                              (:exec . "%c //e:jscript %o %s %a")
                              (:cmdopt . "//Nologo")
-                             (:description . "Run Javascript file with cscript")))
+                             (:description . "Run JavaScript file with cscript")))
+    ("javascript/deno" . ((:command . "deno")
+                          (:exec . "%c run -A %s")
+                          (:compile-only . "%c compile %s")
+                          (:description . "Run JavaScript file with deno")))
 
     ("coffee" . ((:command . "coffee")
                  (:compile-only . "coffee --print %s")
@@ -321,12 +378,18 @@ FMT and ARGS passed `message'."
               (:compile-conf . ((:compilation-mode . nil) (:mode . js-mode)))
               (:description . "Run JSX script")))
 
-    ("typescript" . ((:command . "tsc")
-                     (:exec . ("%c --target es5 --module commonjs %o %s %a" "node %n.js"))
-                     (:compile-only . "%c %o %s %s")
-                     (:compile-conf . ((:compilation-mode . nil) (:mode . js-mode)))
-                     (:remove  . ("%n.js"))
-                     (:description . "Run TypeScript script")))
+    ("typescript/tsc" . ((:command . "tsc")
+                         (:exec . ("%c --target es5 --module commonjs %o %s %a" "node %n.js"))
+                         (:compile-only . "%c %o %s %s")
+                         (:compile-conf . ((:compilation-mode . nil) (:mode . js-mode)))
+                         (:remove  . ("%n.js"))
+                         (:description . "Run TypeScript script")))
+    ("typescript/deno" . ((:command . "deno")
+                          (:exec . "%c run -A %s")
+                          (:compile-only . "%c compile %s")
+                          (:compile-conf . ((:compilation-mode . nil) (:mode . js-mode)))
+                          (:remove  . ("%n.js"))
+                          (:description . "Run TypeScript script with deno")))
 
     ("markdown/Markdown.pl" . ((:command . "Markdown.pl")
                                (:description . "Convert Markdown to HTML with Markdown.pl")))
@@ -404,10 +467,10 @@ FMT and ARGS passed `message'."
                (:description . "Compile rust and execute")))
 
     ("dart/checked" . ((:command . "dart")
-                       (:cmdopt  . "--enable-type-checks")
-                       (:description . "Run dart with '--enable-type-checks' option")))
+                       (:cmdopt  . "--enable-asserts")
+                       (:description . "Run Dart with '--enable-asserts' option")))
     ("dart/production" . ((:command . "dart")
-                          (:description . "Run dart as without '--enable-type-checks' option")))
+                          (:description . "Run Dart WITHOUT '--enable-asserts' option")))
 
     ("elixir" . ((:command . "elixir")
                  (:description . "Run Elixir script")))
@@ -469,8 +532,17 @@ FMT and ARGS passed `message'."
             (:exec . "%c run %o %s %a")
             (:tempfile . nil)
             (:remove "%n")
-            (:description . "Compile and run V programs"))))
+            (:description . "Compile and run V programs")))
 
+    ("zig" . ((:command     . "zig")
+              (:exec        . "%c run %s")
+              (:description . "Run zig file with Zig program")))
+    ("zig/build" . ((:command     . "zig")
+                    (:exec        . "%c build run")
+                    (:description . "Run zig file with built-in package manager")))
+    ("nix" . ((:command . "nix")
+              (:exec    . "%c eval --file %s")
+              (:description . "Evaluate the Nix expression file"))))
   "List of each programming languages information.
 Parameter form is (\"language\" . parameter-alist).  parameter-alist has
 5 keys and those values , :command, :exec, :remove.
@@ -491,7 +563,8 @@ See explanation of quickrun--template-place-holders
 if you set your own language configuration.")
 
 (defvar quickrun-file-alist
-  '(("\\.c\\'" . "c")
+  '(("\\.\\(scpt\\|applescript\\)\\'" . "applescript")
+    ("\\.c\\'" . "c")
     ("\\.\\(cpp\\|cxx\\|C\\|cc\\)\\'" . "c++")
     ("\\.m\\'" . "objc")
     ("\\.cs\\'" . "c#")
@@ -500,9 +573,10 @@ if you set your own language configuration.")
     ("\\.rb\\'" . "ruby")
     ("\\.py\\'" . "python")
     ("\\.php\\'" . "php")
-    ("\\.\\(el\\|elisp\\)\\'" . "emacs")
+    ("\\.\\(el\\|elisp\\)\\'" . "elisp")
     ("\\.\\(lisp\\|lsp\\)\\'" . "lisp")
     ("\\.\\(scm\\|scheme\\)\\'" . "scheme")
+    ("\\.st\\'" . "st/gst")
     ("\\.rkt\\'" . "racket")
     ("\\.js\\'" . "javascript")
     ("\\.clj\\'" . "clojure")
@@ -540,67 +614,74 @@ if you set your own language configuration.")
     ("\\.\\(gpi\\|plt\\)\\'" . "gnuplot")
     ("\\.kt\\'" . "kotlin")
     ("\\.cr\\'" . "crystal")
-    ("\\.v\\'" . "v"))
+    ("\\.v\\'" . "v")
+    ("\\.zig\\'" . "zig"))
   "Alist of (file-regexp . key)")
 
 (defvar quickrun--major-mode-alist
-  '((c-mode . "c")
-    (c++-mode . "c++")
-    (objc-mode . "objc")
-    (csharp-mode . "c#")
-    ((perl-mode cperl-mode) . "perl")
-    (perl6-mode . "perl6")
-    (ruby-mode . "ruby")
-    (python-mode . "python")
-    (php-mode . "php")
-    (emacs-lisp-mode . "emacs")
-    (lisp-mode . "lisp")
-    (scheme-mode . "scheme")
-    (racket-mode . "racket")
-    ((javascript-mode js-mode js2-mode) . "javascript")
-    (clojure-mode . "clojure")
-    (erlang-mode . "erlang")
-    ((ocaml-mode tuareg-mode) . "ocaml")
-    (fsharp-mode . "fsharp")
-    (go-mode . "go")
-    (io-mode . "io")
-    (lua-mode . "lua")
-    (haskell-mode . "haskell")
-    (java-mode . "java")
-    (d-mode . "d")
-    (fortran-mode . "fortran")
-    (markdown-mode . "markdown")
-    (coffee-mode . "coffee")
-    (jsx-mode . "jsx")
-    (typescript-mode . "typescript")
-    (scala-mode . "scala")
-    (groove-mode . "groovy")
-    (haml-mode . "haml")
-    (sass-mode . "sass")
-    ((less-mode less-css-mode) . "less")
-    (sh-mode . "shellscript")
-    (awk-mode . "awk")
-    (rust-mode . "rust")
-    (dart-mode . "dart/checked")
-    (elixir-mode . "elixir")
-    (tcl-mode . "tcl")
-    (swift-mode . "swift")
-    (ats-mode . "ats")
-    (ess-mode . "r")
-    (nim-mode . "nim")
-    (nimscript-mode . "nimscript")
-    (fish-mode . "fish")
-    (julia-mode . "julia")
-    (gnuplot-mode . "gnuplot")
-    (kotlin-mode . "kotlin")
-    (crystal-mode . "crystal")
-    (v-mode . "v"))
+  '((applescript-mode                            . "applescript")
+    (c-mode                                      . "c")
+    (c++-mode                                    . "c++")
+    (objc-mode                                   . "objc")
+    (csharp-mode                                 . "c#")
+    ((perl-mode cperl-mode)                      . "perl")
+    (perl6-mode                                  . "perl6")
+    (ruby-mode                                   . "ruby")
+    (python-mode                                 . "python")
+    ((php-mode php-ts-mode phps-mode)            . "php")
+    (emacs-lisp-mode                             . "elisp")
+    (lisp-mode                                   . "lisp")
+    (scheme-mode                                 . "scheme")
+    (smalltalk-mode                              . "st/gst")
+    (racket-mode                                 . "racket")
+    ((javascript-mode js-mode js2-mode js3-mode) . "javascript")
+    (clojure-mode                                . "clojure")
+    (erlang-mode                                 . "erlang")
+    ((ocaml-mode tuareg-mode)                    . "ocaml")
+    (fsharp-mode                                 . "fsharp")
+    (go-mode                                     . "go")
+    (io-mode                                     . "io")
+    (lua-mode                                    . "lua")
+    (haskell-mode                                . "haskell")
+    (java-mode                                   . "java")
+    (d-mode                                      . "d")
+    (fortran-mode                                . "fortran")
+    (markdown-mode                               . "markdown")
+    (coffee-mode                                 . "coffee")
+    (jsx-mode                                    . "jsx")
+    (typescript-mode                             . "typescript")
+    (scala-mode                                  . "scala")
+    (groove-mode                                 . "groovy")
+    (haml-mode                                   . "haml")
+    (sass-mode                                   . "sass")
+    ((less-mode less-css-mode)                   . "less")
+    (sh-mode                                     . "shellscript")
+    (awk-mode                                    . "awk")
+    (rust-mode                                   . "rust")
+    (rustic-mode                                 . "rust")
+    (dart-mode                                   . "dart/checked")
+    (elixir-mode                                 . "elixir")
+    (tcl-mode                                    . "tcl")
+    (swift-mode                                  . "swift")
+    ((asm-mode nasm-mode masm-mode)              . "asm")
+    (ats-mode                                    . "ats")
+    (ess-mode                                    . "r")
+    (nim-mode                                    . "nim")
+    (nimscript-mode                              . "nimscript")
+    (fish-mode                                   . "fish")
+    (julia-mode                                  . "julia")
+    (gnuplot-mode                                . "gnuplot")
+    (kotlin-mode                                 . "kotlin")
+    (crystal-mode                                . "crystal")
+    (v-mode                                      . "v")
+    (zig-mode                                    . "zig"))
   "Alist of major-mode and langkey")
 
 (defun quickrun--decide-file-type (filename)
   "Decide file type by FILENAME."
   ;; First search by file extension, Second search by major-mode
-  (or (assoc-default filename quickrun-file-alist 'string-match)
+  (or (let (case-fold-search) (assoc-default filename quickrun-file-alist 'string-match))
+      (let ((case-fold-search t)) (assoc-default filename quickrun-file-alist 'string-match))
       (quickrun--find-from-major-mode-alist)))
 
 (defun quickrun--find-from-major-mode-alist ()
@@ -618,8 +699,8 @@ if you set your own language configuration.")
              (format "not found [%s] language information" lang))))
 
 ;;
-;; Compile Only
-;;
+;;; Compile Only
+
 (defun quickrun--check-using-compilation-mode (compile-conf)
   "Not documented."
   (if (not compile-conf)
@@ -632,7 +713,9 @@ if you set your own language configuration.")
 (defun quickrun--pop-to-buffer (buf cb)
   "Not documented."
   (let ((win (selected-window)))
-    (pop-to-buffer buf)
+    (pop-to-buffer buf
+                   `((display-buffer-in-direction)
+                     (dedicated . t)))
     (funcall cb)
     (unless quickrun-focus-p
       (select-window win))))
@@ -650,10 +733,10 @@ if you set your own language configuration.")
              (process-file-shell-command cmd nil t)
              (goto-char (point-min))
              (quickrun--awhen (assoc-default :mode compile-conf)
-                              (funcall it)
-                              (quickrun--pop-to-buffer
-                               (current-buffer) (lambda () (read-only-mode +1)))
-                              (read-only-mode +1)))
+               (funcall it)
+               (quickrun--pop-to-buffer
+                (current-buffer) (lambda () (read-only-mode 1)))
+               (read-only-mode 1)))
            (quickrun--remove-temp-files)))))
 
 (defun quickrun--compilation-finish-func (_buffer _str)
@@ -661,8 +744,8 @@ if you set your own language configuration.")
   (quickrun--remove-temp-files))
 
 ;;
-;; Execute
-;;
+;;; Execute
+
 (defvar quickrun--timeout-timer nil)
 (defvar quickrun--run-in-shell nil)
 
@@ -702,6 +785,34 @@ if you set your own language configuration.")
       (insert output)
       (ansi-color-apply-on-region start (point)))))
 
+(defun quickrun--get-timestamp ()
+  "Return timestamp for quickrun buffer."
+  (substring (current-time-string) 0 19))
+
+(defun quickrun--insert-header (process)
+  "Insert header to PROCESS buffer."
+  (unless quickrun-output-only
+    (with-current-buffer (process-buffer process)
+      (let ((inhibit-read-only t)
+            (time (quickrun--get-timestamp)))
+        (insert "-*- mode: quickrun-; default-directory: \""
+                default-directory
+                "\" -*-\n")
+        (insert "Quickrun started at " time "\n\n")))))
+
+(defun quickrun--insert-footer (process code)
+  "Insert footer to PROCESS buffer with exit CODE."
+  (unless quickrun-output-only
+    (with-current-buffer (process-buffer process)
+      (let ((inhibit-read-only t)
+            (time (quickrun--get-timestamp)))
+        (insert "\n\n")
+        (if (zerop code)
+            (insert "Quickrun finished at " time "\n")
+          (insert "Quickrun exited abnormally with code "
+                  (quickrun-2str code)
+                  " at " time "\n"))))))
+
 (defun quickrun--exec (cmd-lst src mode)
   "Not documented."
   (if quickrun--run-in-shell
@@ -718,7 +829,8 @@ if you set your own language configuration.")
         (when (eq outputter 'quickrun--default-outputter)
           (set-process-filter process #'quickrun--default-filter))
         (set-process-sentinel process
-                              (quickrun--make-sentinel rest-cmds outputter src mode))))))
+                              (quickrun--make-sentinel rest-cmds outputter src mode))
+        (quickrun--insert-header process)))))
 
 (defvar quickrun--eshell-buffer-name "*eshell-quickrun*")
 (defvar quickrun--shell-last-command)
@@ -733,6 +845,12 @@ if you set your own language configuration.")
   (interactive)
   (jump-to-register :quickrun-shell))
 
+(defvar quickrun--eshell-map
+  (let ((map (make-sparse-keymap)))
+    (set-keymap-parent map eshell-mode-map)
+    (define-key map (kbd "q") 'quickrun--eshell-window-restore)
+    map))
+
 (defun quickrun--eshell-post-hook ()
   "Not documented."
   (let ((rerun-p nil)
@@ -745,8 +863,8 @@ if you set your own language configuration.")
               (setq rerun-p t))))
       (unless rerun-p
         (quickrun--eshell-finish)
-        (read-only-mode +1)
-        (local-set-key (kbd "q") 'quickrun--eshell-window-restore)))))
+        (read-only-mode 1)
+        (use-local-map quickrun--eshell-map)))))
 
 (defun quickrun--insert-command (cmd-str)
   "Not documented."
@@ -768,7 +886,7 @@ if you set your own language configuration.")
       (when eshell-buf
         (kill-buffer eshell-buf))
       (eshell)
-      (kill-buffer quickrun--buffer-name)
+      (quickrun--kill-quickrun-buffer)
       (setq-local quickrun--shell-last-command cmd-str)
       (add-hook 'eshell-post-command-hook 'quickrun--eshell-post-hook)
       (quickrun--insert-command cmd-str)
@@ -783,15 +901,15 @@ if you set your own language configuration.")
   "Not documented."
   (let ((cmd-info (quickrun--command-info cmd-key)))
     (quickrun--awhen (assoc-default :default-directory cmd-info)
-                     (let ((formatted (file-name-as-directory it)))
-                       (unless (file-directory-p formatted)
-                         (throw 'quickrun (format "'%s' is not existed directory" it)))
-                       (let* ((has-space (string-match-p "[ \t]" formatted))
-                              (quoted-name (shell-quote-argument
-                                            (if has-space
-                                                (concat "\"" formatted "\"")
-                                              formatted))))
-                         (setq quickrun-option-default-directory quoted-name))))))
+      (let ((formatted (file-name-as-directory it)))
+        (unless (file-directory-p formatted)
+          (throw 'quickrun (format "'%s' is not existed directory" it)))
+        (let* ((has-space (string-match-p "[ \t]" formatted))
+               (quoted-name (shell-quote-argument
+                             (if has-space
+                                 (concat "\"" formatted "\"")
+                               formatted))))
+          (setq quickrun-option-default-directory quoted-name))))))
 
 (defsubst quickrun--process-connection-type (cmd)
   "Not documented."
@@ -826,7 +944,7 @@ if you set your own language configuration.")
                       (process-name process)
                       quickrun-timeout-seconds)))
     (quickrun--remove-temp-files)
-    (quickrun--pop-to-buffer buf (lambda () (read-only-mode +1)))))
+    (quickrun--pop-to-buffer buf (lambda () (read-only-mode 1)))))
 
 (defun quickrun--remove-temp-files ()
   "Remove temporary files."
@@ -854,12 +972,12 @@ if you set your own language configuration.")
 
 (define-derived-mode quickrun--mode nil "Quickrun"
   "Major mode for Quickrun execution process."
-  (read-only-mode +1)
+  (read-only-mode 1)
+  (setq-local truncate-lines quickrun-truncate-lines)
   (use-local-map quickrun--mode-map))
 
 ;;
-;; Predefined outputter
-;;
+;;; Predefined outputter
 
 (defvar quickrun--defined-outputter-symbol
   '((message  . quickrun--outputter-message)
@@ -915,7 +1033,7 @@ if you set your own language configuration.")
 (defun quickrun--outputter-null ()
   "Not documented."
   (delete-region (point-min) (point-max))
-  (kill-buffer (get-buffer quickrun--buffer-name)))
+  (quickrun--kill-quickrun-buffer))
 
 (defun quickrun--outputter-replace-region ()
   "Not documented."
@@ -975,11 +1093,11 @@ if you set your own language configuration.")
         (let ((quickrun--original-buffer origbuf))
           (read-only-mode -1)
           (funcall outputter-func)
-          (read-only-mode +1))))))
+          (read-only-mode 1))))))
 
 (defun quickrun--apply-compilation-mode (input-file mode)
   "Not documented."
-  (when (not (string= input-file quickrun--executed-file))
+  (unless (string= input-file quickrun--executed-file)
     (save-excursion
       (goto-char (point-min))
       (let ((case-fold-search nil))
@@ -996,7 +1114,7 @@ if you set your own language configuration.")
       (read-only-mode -1))
     (quickrun--default-outputter)
     (goto-char (point-min))
-    (read-only-mode +1)))
+    (read-only-mode 1)))
 
 (defun quickrun--make-sentinel (rest-commands outputter-func input orig-mode)
   "Not documented."
@@ -1005,14 +1123,16 @@ if you set your own language configuration.")
     (setq quickrun-option-outputter outputter-func)
     (when (memq (process-status process) '(exit signal))
       (and quickrun--timeout-timer (cancel-timer quickrun--timeout-timer))
-      (delete-process process)
       (let* ((exit-status (process-exit-status process))
              (is-success (zerop exit-status)))
+        (quickrun--insert-footer process exit-status)
+        (delete-process process)
         (cond ((and is-success rest-commands)
                (quickrun--exec rest-commands input orig-mode))
               (t
                (if (not is-success)
-                   (if (eq quickrun-option-outputter #'quickrun--default-outputter)
+                   (if (and (buffer-live-p quickrun--buffer-name)
+                            (eq quickrun-option-outputter #'quickrun--default-outputter))
                        (quickrun--apply-colorizing input orig-mode)
                      (message "Failed: Exit Status=%d" exit-status))
                  (quickrun--apply-outputter outputter-func)
@@ -1023,8 +1143,8 @@ if you set your own language configuration.")
                (quickrun--remove-temp-files)))))))
 
 ;;
-;; Composing command
-;;
+;;; Composing command
+
 (defconst quickrun--template-place-holders
   '("%c" "%o" "%s" "%S" "%a" "%d" "%n" "%N" "%e" "%E")
   "A list of place holders of each language parameter.
@@ -1051,7 +1171,7 @@ Place holders are beginning with '%' and replaced by:
   (let ((buffile (buffer-file-name)))
     (if (not (and buffile (file-remote-p buffile)))
         src
-      (aref (tramp-dissect-file-name (buffer-file-name)) 3))))
+      (tramp-file-name-localname (tramp-dissect-file-name (buffer-file-name))))))
 
 (defun quickrun--place-holder-info (cmd cmdopt source args)
   "Not documented."
@@ -1153,17 +1273,18 @@ Place holders are beginning with '%' and replaced by:
         (setq str (replace-regexp-in-string holder rep str t))))))
 
 ;;
-;; initialize
-;;
+;;; initialize
 
 (defconst quickrun--support-languages
-  '("c" "c++" "objc" "c#" "perl" "perl6" "ruby" "python" "php" "emacs" "lisp" "scheme" "racket"
-    "javascript" "clojure" "erlang" "ocaml" "fsharp" "go" "io" "haskell" "java"
-    "d" "markdown" "coffee" "scala" "groovy" "sass" "less" "shellscript" "awk"
-    "lua" "rust" "dart" "elixir" "tcl" "jsx" "typescript" "fortran" "haml"
-    "swift" "ats" "r" "nim" "nimscript" "fish" "julia" "gnuplot" "kotlin" "crystal", "v")
+  '("asm" "applescript"
+    "c" "c++" "objc" "c#" "perl" "perl6" "ruby" "python" "php" "elisp" "lisp"
+    "scheme" "st" "racket" "javascript" "clojure" "erlang" "ocaml" "fsharp"
+    "go" "io" "haskell" "java" "d" "markdown" "coffee" "scala" "groovy" "sass"
+    "less" "shellscript" "awk" "lua" "rust" "dart" "elixir" "tcl" "jsx"
+    "typescript" "fortran" "haml" "swift" "ats" "r" "nim" "nimscript" "fish"
+    "julia" "gnuplot" "kotlin" "crystal" "v" "zig")
   "Programming languages and Markup languages supported as default
-by quickrun.el. But you can register your own command for some languages")
+by quickrun.el.  But you can register your own command for some languages")
 
 (defvar quickrun--command-key-table
   (make-hash-table :test 'equal))
@@ -1175,6 +1296,24 @@ by quickrun.el. But you can register your own command for some languages")
     (error "%s is not registered" key))
   (puthash lang key quickrun--command-key-table))
 
+;;;###autoload
+(defun quickrun-select-default ()
+  "Update the default."
+  (interactive)
+  (when-let*
+      ((src (buffer-name))
+       (lang (or (and src (quickrun--decide-file-type src))
+                 (quickrun--find-from-major-mode-alist)))
+       (default (ht-get quickrun--command-key-table lang))
+       (candidates (cl-remove-if-not (lambda (item)
+                                       (string= lang (nth 0 (split-string (car item) "/"))))
+                                     quickrun--language-alist))
+       (prompt (format "QuickRun Lang%s: "(if default
+                                              (format " [Default: %s]" default)
+                                            "")))
+       (key (completing-read prompt candidates nil nil nil nil default)))
+    (quickrun-set-default lang key)))
+
 (defun quickrun--override-command (cmdkey cmd-alist)
   "Not documented."
   (let ((registered (assoc-default cmdkey quickrun--language-alist)))
@@ -1213,7 +1352,7 @@ by quickrun.el. But you can register your own command for some languages")
 (defun quickrun--set-command-key (lang candidates)
   "Not documented."
   (quickrun--awhen (quickrun--find-executable candidates)
-                   (puthash lang (format "%s/%s" lang it) quickrun--command-key-table)))
+    (puthash lang (format "%s/%s" lang it) quickrun--command-key-table)))
 
 (defsubst quickrun--c-compiler ()
   "Not documented."
@@ -1228,18 +1367,22 @@ by quickrun.el. But you can register your own command for some languages")
         (t '("g++" "clang++"))))
 
 (defconst quicklang/lang-candidates
-  `(("c" . ,(quickrun--c-compiler))
+  `(("asm" . ("nasm" "masm" "masm64"))
+    ("c" . ,(quickrun--c-compiler))
     ("c++" . ,(quickrun--c++-compiler))
     ("c#" . ("dotnet" "mono"))
+    ("elisp" . ("emacs" "eask" "eask-g" "eask-c"))
     ("fortran" . ("gfortran"))
-    ("javascript" . ("node" "v8" "js" "jrunscript" "cscript"))
+    ("javascript" . ("node" "v8" "js" "jrunscript" "cscript" "deno"))
     ("ruby" . ("ruby" "mruby"))
     ("lisp" . ("clisp" "sbcl" "ccl"))
     ("scheme" . ("gosh"))
     ("swift" . ("xcrun" "swift"))
+    ("typescript" . ("tsc" "deno"))
     ("markdown" . ("Markdown.pl" "kramdown" "bluecloth" "redcarpet" "pandoc"))
     ("clojure" . ("jark" "clj-env-dir"))
-    ("go" . ("go" "gccgo")))
+    ("go" . ("go" "gccgo"))
+    ("zig" . ("zig" "build")))
   "Candidates of language which has some compilers or interpreters.")
 
 (defun quickrun--init-command-key-table ()
@@ -1270,13 +1413,14 @@ by quickrun.el. But you can register your own command for some languages")
               (file-name-nondirectory buffer-file))))))
 
 ;;
-;; main
-;;
+;;; main
+
 ;;;###autoload
 (defun quickrun (&rest plist)
-  "Run commands quickly for current buffer
-   With universal prefix argument(C-u), select command-key,
-   With double prefix argument(C-u C-u), run in compile-only-mode."
+  "Run commands quickly for current buffer.
+
+With universal prefix argument(C-u), select command-key,
+With double prefix argument(C-u C-u), run in compile-only-mode."
   (interactive)
   (quickrun--set-executed-file)
   (let ((beg (or (plist-get plist :start) (point-min)))
@@ -1295,6 +1439,12 @@ by quickrun.el. But you can register your own command for some languages")
 
 (defvar quickrun--with-arg--history nil)
 
+;;;###autoload
+(defun quickrun-select ()
+  "Run commands after selecting the backend."
+  (interactive)
+  (when (quickrun-select-default) (quickrun)))
+
 ;;;###autoload
 (defun quickrun-with-arg (arg)
   "Run commands quickly for current buffer with arguments."
@@ -1309,7 +1459,7 @@ by quickrun.el. But you can register your own command for some languages")
   "Not documented."
   (let* ((default (or quickrun-option-cmdkey quickrun--last-cmd-key))
          (prompt (format "QuickRun Lang%s: "(if default
-                                                (format "[Default: %s]" default)
+                                                (format " [Default: %s]" default)
                                               ""))))
     (completing-read prompt quickrun--language-alist nil nil nil nil default)))
 
@@ -1347,11 +1497,19 @@ by quickrun.el. But you can register your own command for some languages")
   (let ((quickrun--compile-only-flag t))
     (quickrun)))
 
+;;;###autoload
+(defun quickrun-compile-only-select ()
+  "Run commands after selecting the backend."
+  (interactive)
+  (let ((quickrun--compile-only-flag t))
+    (quickrun-select)))
+
 ;;;###autoload
 (defun quickrun-shell ()
   "Run commands in shell for interactive programs."
   (interactive)
   (let ((quickrun--run-in-shell t)
+        (quickrun-focus-p t)
         (quickrun-timeout-seconds nil))
     (quickrun)))
 
@@ -1369,13 +1527,14 @@ by quickrun.el. But you can register your own command for some languages")
 
 (defun quickrun--command-key (src)
   "Not documented."
-  (let ((file-type (and src (quickrun--decide-file-type src)))
+  (let ((file-type (or (and src (quickrun--decide-file-type src))
+                       (quickrun--find-from-major-mode-alist)))
         (use-prefix-p (and (consp current-prefix-arg)
                            (= (car current-prefix-arg) 4))))
     (or (and use-prefix-p (quickrun--prompt))
         (and quickrun-option-cmd-alist "_user_defined") ;; setting dummy value
         quickrun-option-cmdkey
-        (and (not src) (quickrun--prompt))
+        (and (not file-type) (quickrun--prompt))
         (gethash file-type quickrun--command-key-table)
         file-type
         (quickrun--prompt))))
@@ -1415,7 +1574,7 @@ by quickrun.el. But you can register your own command for some languages")
       (let* ((cmdinfo (quickrun--command-info cmd-key))
              (tempfile-param (assoc :tempfile cmdinfo)))
         (if tempfile-param
-            (cdr tempfile-param)
+            (cadr tempfile-param)
           t)))))
 
 (defsubst quickrun--buffer-popup-p ()
@@ -1428,7 +1587,6 @@ by quickrun.el. But you can register your own command for some languages")
   (let* ((orig-src quickrun--executed-file)
          (cmd-key (quickrun--command-key orig-src)))
     (quickrun--set-default-directory cmd-key)
-    (quickrun--kill-quickrun-buffer)
     (unless (local-variable-p 'quickrun--last-cmd-key)
       (make-local-variable 'quickrun--last-cmd-key))
     (setq quickrun--last-cmd-key cmd-key)
@@ -1477,8 +1635,7 @@ by quickrun.el. But you can register your own command for some languages")
     (remove-hook 'after-save-hook 'quickrun--without-focus t)))
 
 ;;
-;; helm/anything interface
-;;
+;;; helm/anything interface
 
 (defconst helm-quickrun--actions
   '(("Run this cmd-key" . quickrun--helm-action-default)
